//----------------------------------------------------------------------------------
// File:   SolidWireframe.cpp
// Author: Samuel Gateau
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

#include "DXUT.h"
#include "CkTools.h"
#include "sdkmesh_old.h"

//--------------------------------------------------------------------------------------
// Global Variables
//--------------------------------------------------------------------------------------

CkDemoWindow g_DemoWindow;

// The effect
ID3D10Effect* g_pEffect;

// The input layout.
D3D10_INPUT_ELEMENT_DESC g_pInputElements[] =
{
    { "POSITION", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 0, D3D10_INPUT_PER_VERTEX_DATA, 0 },
    { "NORMAL", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 12, D3D10_INPUT_PER_VERTEX_DATA, 0 },
    { "TEXCOORD", 0, DXGI_FORMAT_R32G32_FLOAT, 0, 24, D3D10_INPUT_PER_VERTEX_DATA, 0 },
};
ID3D10InputLayout* g_pVertexLayout;

// The geometry
std::vector<CDXUTMesh10*> g_Geometries;
int g_CurrentGeometry = 0;


enum TechniqueModes
{
    eDisplayDepthAndShadedPass = 0,
    eDisplayDepthAndShadedBiasedPass,
    eDisplayDepthPass,
    eDisplayDepthBiasedPass,
    eDisplayShadedPass,
    eNbTechniqueModes,
};

int g_TechniqueNbOffset = eNbTechniqueModes;
int g_NbTechniques = 0;
bool g_DisplayDepth = true;
bool g_DisplayDepthBiased = true;
bool g_DisplayShaded = true;
bool g_DisplayTechnique = true;

ID3D10EffectScalarVariable* g_pLineWidthVariable = NULL;
ID3D10EffectScalarVariable* g_pFadeDistanceVariable = NULL;
ID3D10EffectScalarVariable* g_pPatternPeriodVariable = NULL;

ID3D10EffectMatrixVariable* g_pWorldVariable;
ID3D10EffectMatrixVariable* g_pViewVariable ;
ID3D10EffectMatrixVariable* g_pProjectionVariable ;
ID3D10EffectMatrixVariable* g_pWorldViewVariable ;
ID3D10EffectMatrixVariable* g_pWorldViewProjectionVariable ;
ID3D10EffectVectorVariable* g_pFrustumVariable ;
ID3D10EffectVectorVariable* g_pViewportVariable ;

HRESULT createEffect(ID3D10Device* pd3dDevice);
HRESULT createLayout(ID3D10Device* pd3dDevice);
HRESULT createMesh(ID3D10Device* pd3dDevice, LPCWSTR pFileName = L"..//..//Media//hydrant.x");

HRESULT WINAPI NVUTFindDXSDKMediaFileCch( WCHAR* strDestPath, int cchDest, LPCWSTR strFilename )
{
	HRESULT hr = DXUTFindDXSDKMediaFileCch( strDestPath, cchDest, strFilename );

	if (FAILED(hr))
	{
		WCHAR strFullError[MAX_PATH] = L"The file ";
		StringCchCat( strFullError, MAX_PATH, strFilename );
		StringCchCat( strFullError, MAX_PATH, L" was not found on the search path.\nCannot continue - exiting." );

		MessageBoxW(NULL, strFullError, L"Media file not found", MB_OK);
		exit(0);
	}

	return hr;
}

//--------------------------------------------------------------------------------------
// Create any D3D10 resources that aren't dependant on the back buffer
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnD3D10CreateDevice( ID3D10Device* pd3dDevice, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
{
    HRESULT hr;
    hr = g_DemoWindow.OnD3D10CreateDevice(pd3dDevice, pBackBufferSurfaceDesc);
    if( FAILED(hr) ) return hr;

    hr = createEffect(pd3dDevice);
    if( FAILED(hr) ) return hr;

    hr = createLayout(pd3dDevice);
    if( FAILED(hr) ) return hr;

    hr = createMesh(pd3dDevice);
    if( FAILED(hr) ) return hr;

    D3DXVECTOR3 inspect(0, 0 , 0);
    D3DXVECTOR3 euler(0, -0.5, 0);
    g_DemoWindow.mCamera.setViewFrame( inspect, euler, 250 );

    return S_OK;
}


//--------------------------------------------------------------------------------------
// Create any D3D10 resources that depend on the back buffer
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnD3D10ResizedSwapChain( ID3D10Device* pd3dDevice, IDXGISwapChain *pSwapChain, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
{
    g_DemoWindow.OnD3D10ResizedSwapChain(pd3dDevice, pSwapChain, pBackBufferSurfaceDesc);    
    return S_OK;
}

//--------------------------------------------------------------------------------------
// Render the scene using the D3D10 device
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10FrameRender( ID3D10Device* pd3dDevice, double fTime, float fElapsedTime, void* pUserContext )
{
    // Clear render target and the depth stencil 
    float ClearColor[4] = { 0.5f, 0.5f, 0.5f, 0.f };
    pd3dDevice->ClearRenderTargetView( DXUTGetD3D10RenderTargetView(), ClearColor );
    pd3dDevice->ClearDepthStencilView( DXUTGetD3D10DepthStencilView(), D3D10_CLEAR_DEPTH, 1.0, 0 );

    // Update the projection matrix using the very last viewport state.
    g_DemoWindow.mCamera.setProjectionFrame( D3DX_PI/3, g_DemoWindow.getViewportAspectRatio(0), 1.0f, 1000.0f );
    g_pProjectionVariable->SetMatrix( (float*)&g_DemoWindow.mCamera.getProjectionFrame() );
    float frustumf[4];
    frustumf[0] = g_DemoWindow.mCamera.getWidth();
    frustumf[1] = g_DemoWindow.mCamera.getHeight();
    frustumf[2] = g_DemoWindow.mCamera.getNear();
    frustumf[3] = g_DemoWindow.mCamera.getFar();
    g_pFrustumVariable->SetFloatVector((float*) &frustumf);

    // Update matrix variables
    g_pWorldVariable->SetMatrix( (float*)&g_DemoWindow.mCamera.getWorldFrame() );
    g_pViewVariable->SetMatrix( (float*)&g_DemoWindow.mCamera.getViewFrame() );

    D3DXMATRIX tempMat;
    g_pWorldViewVariable->SetMatrix( (float*) &g_DemoWindow.mCamera.getWorldViewFrame(tempMat) );
    g_pWorldViewProjectionVariable->SetMatrix( (float*) &g_DemoWindow.mCamera.getWorldViewProjectionFrame(tempMat) );

    // Render
    for (int t = g_TechniqueNbOffset; t < g_NbTechniques + g_TechniqueNbOffset; ++t)
    {
        if (    (g_DemoWindow.mSelectedViewport < 0)
            ||  (t == g_DemoWindow.mSelectedViewport + g_TechniqueNbOffset) )
        {
            int viewportId = t-g_TechniqueNbOffset;
            // Update the viewport to draw in the next viewport. 
            D3D10_VIEWPORT vp;
            vp.Width = static_cast<UINT> (g_DemoWindow.getViewportWidth(viewportId));
            vp.Height = static_cast<UINT> (g_DemoWindow.getViewportHeight(viewportId));
            vp.MinDepth = 0;
            vp.MaxDepth = 1;
            vp.TopLeftX = static_cast<INT> (g_DemoWindow.getViewportTopLeftX(viewportId));
            vp.TopLeftY = static_cast<INT> (g_DemoWindow.getViewportTopLeftY(viewportId));
            pd3dDevice->RSSetViewports( 1, &vp );

            float viewportf[4];
            viewportf[0] = g_DemoWindow.getViewportWidth(viewportId);
            viewportf[1] = g_DemoWindow.getViewportHeight(viewportId);
            viewportf[2] = g_DemoWindow.getViewportTopLeftX(viewportId);
            viewportf[3] = g_DemoWindow.getViewportTopLeftY(viewportId);
            g_pViewportVariable->SetFloatVector( (float*)&viewportf );

            //Draw the depth and shaded solid pass
            if (g_DisplayDepth && g_DisplayShaded)
            {
                // apply the biased version of the depth pass for the DX wireframe rendering.
                UINT tidx = (g_DisplayDepthBiased && (t == eNbTechniqueModes) ? eDisplayDepthAndShadedBiasedPass : eDisplayDepthAndShadedPass);
                g_Geometries[g_CurrentGeometry]->Render(pd3dDevice, g_pEffect->GetTechniqueByIndex(tidx));
            }
            // Or Draw the depth pass 
            else if (g_DisplayDepth)
            {
                // apply the biased version of the depth pass for the DX wireframe rendering.
                UINT tidx = (g_DisplayDepthBiased && (t == eNbTechniqueModes) ? eDisplayDepthBiasedPass : eDisplayDepthPass);
                g_Geometries[g_CurrentGeometry]->Render(pd3dDevice, g_pEffect->GetTechniqueByIndex(tidx));
            }
            // Or Draw the shaded solid pass 
            else if (g_DisplayShaded)
            {
                g_Geometries[g_CurrentGeometry]->Render(pd3dDevice, g_pEffect->GetTechniqueByIndex(eDisplayShadedPass));
            }

            // Draw the Technique
            if (g_DisplayTechnique)
            {
                g_Geometries[g_CurrentGeometry]->Render(pd3dDevice, g_pEffect->GetTechniqueByIndex(t));
            }
        }
    }
    g_DemoWindow.OnD3D10FrameRender(pd3dDevice, fTime, fElapsedTime);
}


//--------------------------------------------------------------------------------------
// Release D3D10 resources created in OnD3D10ResizedSwapChain 
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10ReleasingSwapChain( void* pUserContext )
{
    g_DemoWindow.OnD3D10ReleasingSwapChain();
}

void destroyGeometries()
{
    for (unsigned int meshNb = 0; meshNb < g_Geometries.size(); ++meshNb)
    {
        g_Geometries[meshNb]->Destroy();
        g_Geometries[meshNb] = NULL;
    }
    g_Geometries.clear();
}

//--------------------------------------------------------------------------------------
// Release D3D10 resources created in OnD3D10CreateDevice 
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10DestroyDevice( void* pUserContext )
{
    g_DemoWindow.OnD3D10DestroyDevice();

    SAFE_RELEASE( g_pEffect );
    SAFE_RELEASE( g_pVertexLayout );

    destroyGeometries();
}

//--------------------------------------------------------------------------------------
// Handle messages to the application
//--------------------------------------------------------------------------------------
LRESULT CALLBACK MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, 
                          bool* pbNoFurtherProcessing, void* pUserContext )
{
    *pbNoFurtherProcessing = g_DemoWindow.MsgProc( hWnd, uMsg, wParam, lParam );
    return *pbNoFurtherProcessing;
}
//--------------------------------------------------------------------------------------
// Callback called when the reset button is pressed.
//--------------------------------------------------------------------------------------
void OnReset()
{
	// Reset parameters
}

//--------------------------------------------------------------------------------------
// Callback called when the load model button is pressed.
//--------------------------------------------------------------------------------------
void OnLoadModel()
{
    HRESULT hr;

    WCHAR fileName[MAX_PATH] = {0};

    OPENFILENAME info;
    ZeroMemory(&info, sizeof (OPENFILENAME));
    info.lStructSize = sizeof (OPENFILENAME);
    info.hwndOwner = DXUTGetHWND();
    info.lpstrFilter = L"X file meshes (.x)\0*.x\0\0";
    info.nFilterIndex = 0;
    info.lpstrFile = fileName;
    info.nMaxFile = MAX_PATH;
    info.lpstrTitle = L"Open model";
    info.nMaxFileTitle = sizeof(info.lpstrTitle);
    info.dwReserved = OFN_EXPLORER;

    if (GetOpenFileName(&info))
    {
        hr = createMesh(DXUTGetD3D10Device(), fileName);
        if (FAILED(hr))
        {
            MessageBox(NULL, L"Couldn't load the file", NULL, MB_ICONSTOP);
        }

        g_DemoWindow.mCamera.resetViewFrame();
    }
}

//--------------------------------------------------------------------------------------
// Initialize everything and go into a render loop
//--------------------------------------------------------------------------------------
int WINAPI wWinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nCmdShow )
{
    // Enable run-time memory check for debug builds.
#if defined(DEBUG) | defined(_DEBUG)
    _CrtSetDbgFlag( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
#endif

    // DXUT will create and use the best device (either D3D9 or D3D10) 
    // that is available on the system depending on which D3D callbacks are set below

    // Set DXUT callbacks
    DXUTSetCallbackMsgProc( MsgProc );
    DXUTSetCallbackD3D10DeviceCreated( OnD3D10CreateDevice );
    DXUTSetCallbackD3D10SwapChainResized( OnD3D10ResizedSwapChain );
    DXUTSetCallbackD3D10FrameRender( OnD3D10FrameRender );
    DXUTSetCallbackD3D10SwapChainReleasing( OnD3D10ReleasingSwapChain );
    DXUTSetCallbackD3D10DeviceDestroyed( OnD3D10DestroyDevice );

    g_DemoWindow.setResetCallback( (CkDemoWindow::CkResetCallback*) OnReset );
    g_DemoWindow.setLoadModelCallback( (CkDemoWindow::CkLoadModelCallback*) OnLoadModel );

    // IMPORTANT: set SDK media search path to include source directory of this sample, when started from .\Bin
    HRESULT hr;
    V_RETURN(DXUTSetMediaSearchPath(L"..//Source//SolidWireframe"));

    DXUTInit( true, true, NULL ); // Parse the command line, show msgboxes on error, no extra command line params
    DXUTSetCursorSettings( true, true ); // Show the cursor and clip it when in full screen
    DXUTCreateWindow( L"SolidWireframe" );
    DXUTCreateDevice( true, 640, 480 );  
    DXUTMainLoop(); // Enter into the DXUT render loop

    // Perform any application-level cleanup here

    return DXUTGetExitCode();
}


HRESULT createEffect(ID3D10Device* pd3dDevice)
{
    HRESULT hr;

    // Create the effect
    ID3D10Blob *pErrors = NULL;
    WCHAR fullPath[MAX_PATH];
    V_RETURN( NVUTFindDXSDKMediaFileCch(fullPath, MAX_PATH, L"SolidWireframe.fx") );
    hr = D3DX10CreateEffectFromFile( fullPath, NULL, NULL, "fx_4_0", D3D10_SHADER_ENABLE_STRICTNESS, 0, pd3dDevice, NULL, NULL, &g_pEffect, NULL, &hr );
    if( FAILED( hr ) )
    {
        LPCSTR szErrors = NULL;
        if(pErrors)
        {
            szErrors = (LPCSTR)pErrors->GetBufferPointer();
            OutputDebugStringA(szErrors);
            pErrors->Release();
        }
        return E_FAIL;
    }

    // Remember the number of relevant techniques
    D3D10_EFFECT_DESC effectDesc; 
    g_pEffect->GetDesc( &effectDesc );	
    g_NbTechniques = effectDesc.Techniques - g_TechniqueNbOffset;

    // Configure the viewports for the number of techniques.
    g_DemoWindow.setNbViewports(g_NbTechniques);

    // Use the technique name and annotations to describe the effect when viewport is selected.
    // Render
    for (int t = g_TechniqueNbOffset; t < g_NbTechniques + g_TechniqueNbOffset; ++t)
    {
        D3D10_TECHNIQUE_DESC lDesc;
        ID3D10EffectVariable* lVar = NULL;
        g_pEffect->GetTechniqueByIndex(t)->GetDesc(&lDesc);
        if (lDesc.Annotations > 0)
        {
            lVar = g_pEffect->GetTechniqueByIndex(t)->GetAnnotationByName("info");
        }

        if (lVar)
        {
            LPCSTR info;
            lVar->AsString()->GetString(&info);
            g_DemoWindow.mViewportTitles.push_back(stringtowstring(std::string(info), std::wstring()));
        }
        else
        {
            g_DemoWindow.mViewportTitles.push_back(stringtowstring(std::string(lDesc.Name), std::wstring()));
        }
    }

    // Obtain the variables
    g_pWorldVariable = g_pEffect->GetVariableByName( "World" )->AsMatrix();
    g_pViewVariable = g_pEffect->GetVariableByName( "View" )->AsMatrix();
    g_pProjectionVariable = g_pEffect->GetVariableByName( "Projection" )->AsMatrix();
    g_pWorldViewVariable = g_pEffect->GetVariableByName( "WorldView" )->AsMatrix();
    g_pWorldViewProjectionVariable = g_pEffect->GetVariableByName( "WorldViewProjection" )->AsMatrix();
    g_pFrustumVariable = g_pEffect->GetVariableByName( "Frustum" )->AsVector();
    g_pViewportVariable = g_pEffect->GetVariableByName( "Viewport" )->AsVector();

    g_pLineWidthVariable = g_pEffect->GetVariableByName( "LineWidth" )->AsScalar();
    g_pFadeDistanceVariable = g_pEffect->GetVariableByName( "FadeDistance" )->AsScalar();
    g_pPatternPeriodVariable = g_pEffect->GetVariableByName( "PatternPeriod" )->AsScalar();	

    // Create editor for main global vars
    new CkDemoWindow::CkPropertyBool(g_DemoWindow, &g_DisplayDepth, std::wstring(L"Draw depth"));
    new CkDemoWindow::CkPropertyBool(g_DemoWindow, &g_DisplayDepthBiased, std::wstring(L"Bias depth on DXWire"));
    new CkDemoWindow::CkPropertyBool(g_DemoWindow, &g_DisplayShaded, std::wstring(L"Draw shaded"));
    new CkDemoWindow::CkPropertyBool(g_DemoWindow, &g_DisplayTechnique, std::wstring(L"Draw wireframe technique"));

    // Create variables ui:
    new CkDemoWindow::CkPropertyFXScalar(g_DemoWindow, g_pLineWidthVariable, 0.f, 10.f, std::wstring(L"Line width [pxl]"));
    new CkDemoWindow::CkPropertyFXScalar(g_DemoWindow, g_pFadeDistanceVariable, 0.1f, 100.f, std::wstring(L"Fade distance"));
    new CkDemoWindow::CkPropertyFXScalar(g_DemoWindow, g_pPatternPeriodVariable, 1.f, 20.f, std::wstring(L"Pattern Period"));

    return hr;
}

HRESULT createLayout(ID3D10Device* pd3dDevice)
{
    HRESULT hr;

    UINT numElements = sizeof(g_pInputElements)/sizeof(g_pInputElements[0]);

    // Create the input layout
    D3D10_PASS_DESC PassDesc;
    g_pEffect->GetTechniqueByIndex(0)->GetPassByIndex( 0 )->GetDesc( &PassDesc );

    V_RETURN( pd3dDevice->CreateInputLayout( g_pInputElements, numElements, PassDesc.pIAInputSignature, PassDesc.IAInputSignatureSize, &g_pVertexLayout ) );

    // Set the input layout
    pd3dDevice->IASetInputLayout( g_pVertexLayout );

    // Set primitive topology
    pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST );

    return hr;
}

HRESULT createMesh(ID3D10Device* pd3dDevice, LPCWSTR pFileName)
{
    HRESULT hr;

    // Clean current geometry
    destroyGeometries();

    // THen add new geometries according to the filename.
    CDXUTMesh10* mesh = NULL;
    WCHAR fullPath[MAX_PATH];
    
    V_RETURN( NVUTFindDXSDKMediaFileCch(fullPath, MAX_PATH, pFileName) );

    mesh = new CDXUTMesh10();
    V_RETURN( mesh->Create( pd3dDevice, fullPath, (D3D10_INPUT_ELEMENT_DESC*)g_pInputElements, 3 ) );
    g_Geometries.push_back(mesh);

    return hr;
}